/*
*  Copyright 2019-2020 Zheng Jie
*
*  Licensed under the Apache License, Version 2.0 (the "License");
*  you may not use this file except in compliance with the License.
*  You may obtain a copy of the License at
*
*  http://www.apache.org/licenses/LICENSE-2.0
*
*  Unless required by applicable law or agreed to in writing, software
*  distributed under the License is distributed on an "AS IS" BASIS,
*  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*  See the License for the specific language governing permissions and
*  limitations under the License.
*/
package me.zhengjie.modules.dm.foodPj.rest;

import me.zhengjie.annotation.Log;
import me.zhengjie.modules.dm.foodPj.domain.DmFoodPj;
import me.zhengjie.modules.dm.foodPj.service.DmFoodPjService;
import me.zhengjie.modules.dm.foodPj.service.dto.DmFoodPjQueryCriteria;
import org.springframework.data.domain.Pageable;
import lombok.RequiredArgsConstructor;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.*;
import java.io.IOException;
import javax.servlet.http.HttpServletResponse;

/**
* @website https://el-admin.vip
* @author sunmz
* @date 2021-09-15
**/
@RestController
@RequiredArgsConstructor
@Api(tags = "foodPj管理")
@RequestMapping("/api/dmFoodPj")
public class DmFoodPjController {

    private final DmFoodPjService dmFoodPjService;

    @Log("导出数据")
    @ApiOperation("导出数据")
    @GetMapping(value = "/download")
    @PreAuthorize("@el.check('dmFoodPj:list')")
    public void download(HttpServletResponse response, DmFoodPjQueryCriteria criteria) throws IOException {
        dmFoodPjService.download(dmFoodPjService.queryAll(criteria), response);
    }

    @GetMapping
    @Log("查询foodPj")
    @ApiOperation("查询foodPj")
    @PreAuthorize("@el.check('dmFoodPj:list')")
    public ResponseEntity<Object> query(DmFoodPjQueryCriteria criteria, Pageable pageable){
        return new ResponseEntity<>(dmFoodPjService.queryAll(criteria,pageable),HttpStatus.OK);
    }

    @PostMapping
    @Log("新增foodPj")
    @ApiOperation("新增foodPj")
    @PreAuthorize("@el.check('dmFoodPj:add')")
    public ResponseEntity<Object> create(@Validated @RequestBody DmFoodPj resources){
        return new ResponseEntity<>(dmFoodPjService.create(resources),HttpStatus.CREATED);
    }

    @PutMapping
    @Log("修改foodPj")
    @ApiOperation("修改foodPj")
    @PreAuthorize("@el.check('dmFoodPj:edit')")
    public ResponseEntity<Object> update(@Validated @RequestBody DmFoodPj resources){
        dmFoodPjService.update(resources);
        return new ResponseEntity<>(HttpStatus.NO_CONTENT);
    }

    @Log("删除foodPj")
    @ApiOperation("删除foodPj")
    @PreAuthorize("@el.check('dmFoodPj:del')")
    @DeleteMapping
    public ResponseEntity<Object> delete(@RequestBody Long[] ids) {
        dmFoodPjService.deleteAll(ids);
        return new ResponseEntity<>(HttpStatus.OK);
    }
}