/*
*  Copyright 2019-2020 Zheng Jie
*
*  Licensed under the Apache License, Version 2.0 (the "License");
*  you may not use this file except in compliance with the License.
*  You may obtain a copy of the License at
*
*  http://www.apache.org/licenses/LICENSE-2.0
*
*  Unless required by applicable law or agreed to in writing, software
*  distributed under the License is distributed on an "AS IS" BASIS,
*  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*  See the License for the specific language governing permissions and
*  limitations under the License.
*/
package me.zhengjie.modules.dm.userInformation.rest;

import me.zhengjie.annotation.Log;
import me.zhengjie.modules.dm.userInformation.domain.DmUserInformation;
import me.zhengjie.modules.dm.userInformation.service.DmUserInformationService;
import me.zhengjie.modules.dm.userInformation.service.dto.DmUserInformationQueryCriteria;
import org.springframework.data.domain.Pageable;
import lombok.RequiredArgsConstructor;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.*;
import java.io.IOException;
import javax.servlet.http.HttpServletResponse;

/**
* @website https://el-admin.vip
* @author lr
* @date 2022-06-06
**/
@RestController
@RequiredArgsConstructor
@Api(tags = "发布信息记录管理")
@RequestMapping("/api/dmUserInformation")
public class DmUserInformationController {

    private final DmUserInformationService dmUserInformationService;

    @Log("导出数据")
    @ApiOperation("导出数据")
    @GetMapping(value = "/download")
    @PreAuthorize("@el.check('dmUserInformation:list')")
    public void download(HttpServletResponse response, DmUserInformationQueryCriteria criteria) throws IOException {
        dmUserInformationService.download(dmUserInformationService.queryAll(criteria), response);
    }

    @GetMapping
    @Log("查询发布信息记录")
    @ApiOperation("查询发布信息记录")
    @PreAuthorize("@el.check('dmUserInformation:list')")
    public ResponseEntity<Object> query(DmUserInformationQueryCriteria criteria, Pageable pageable){
        return new ResponseEntity<>(dmUserInformationService.queryAll(criteria,pageable),HttpStatus.OK);
    }

    @PostMapping
    @Log("新增发布信息记录")
    @ApiOperation("新增发布信息记录")
    @PreAuthorize("@el.check('dmUserInformation:add')")
    public ResponseEntity<Object> create(@Validated @RequestBody DmUserInformation resources){
        return new ResponseEntity<>(dmUserInformationService.create(resources),HttpStatus.CREATED);
    }

    @PutMapping
    @Log("修改发布信息记录")
    @ApiOperation("修改发布信息记录")
    @PreAuthorize("@el.check('dmUserInformation:edit')")
    public ResponseEntity<Object> update(@Validated @RequestBody DmUserInformation resources){
        dmUserInformationService.update(resources);
        return new ResponseEntity<>(HttpStatus.NO_CONTENT);
    }

    @Log("删除发布信息记录")
    @ApiOperation("删除发布信息记录")
    @PreAuthorize("@el.check('dmUserInformation:del')")
    @DeleteMapping
    public ResponseEntity<Object> delete(@RequestBody Long[] ids) {
        dmUserInformationService.deleteAll(ids);
        return new ResponseEntity<>(HttpStatus.OK);
    }
}